library Webshots;

uses
  DateUtils,
  SysUtils,
  Classes,
  Messages,
  Windows,
  uWebshots in 'uWebshots.pas';

{$E wcx}

const       {Error codes returned to calling application}
  E_END_ARCHIVE=     10;       {No more files in archive}
  E_NO_MEMORY=       11;       {Not enough memory}
  E_BAD_DATA=        12;       {Data is bad}
  E_BAD_ARCHIVE=     13;       {CRC error in archive data}
  E_UNKNOWN_FORMAT=  14;       {Archive format unknown}
  E_EOPEN=           15;       {Cannot open existing file}
  E_ECREATE=         16;       {Cannot create file}
  E_ECLOSE=          17;       {Error closing file}
  E_EREAD=           18;       {Error reading from file}
  E_EWRITE=          19;       {Error writing to file}
  E_SMALL_BUF=       20;       {Buffer too small}
  E_EABORTED=        21;       {Function aborted by user}
  E_NO_FILES=        22;       {No files found}
  E_TOO_MANY_FILES=  23;       {Too many files to pack}
  E_NOT_SUPPORTED=   24;       {Function not supported}

  {Unpacking flags}
  PK_OM_LIST=           0;
  PK_OM_EXTRACT=        1;

  {Flags for ProcessFile}
  PK_SKIP=              0;     {Skip file (no unpacking)}
  PK_TEST=              1;     {Test file integrity}
  PK_EXTRACT=           2;     {Extract file to disk}

  {Flags passed through ChangeVolProc}
  PK_VOL_ASK=           0;     {Ask user for location of next volume}
  PK_VOL_NOTIFY=        1;     {Notify app that next volume will be unpacked}

  {Packing flags}

  {For PackFiles}
  PK_PACK_MOVE_FILES=   1;    {Delete original after packing}
  PK_PACK_SAVE_PATHS=   2;    {Save path names of files}

  {Returned by GetPackCaps}
  PK_CAPS_NEW=          1;    {Can create new archives}
  PK_CAPS_MODIFY=       2;    {Can modify exisiting archives}
  PK_CAPS_MULTIPLE=     4;    {Archive can contain multiple files}
  PK_CAPS_DELETE=       8;    {Can delete files}
  PK_CAPS_OPTIONS=     16;    {Supports the options dialogbox}
  PK_CAPS_MEMPACK=     32;    {Supports packing in memory}
  PK_CAPS_BY_CONTENT=  64;    {Detect archive type by content}
  PK_CAPS_SEARCHTEXT= 128;    {Allow searching for text in archives created with this plugin}
  PK_CAPS_HIDE=       256;    {Don't show as plugin, needs to open with Ctrl+PgDn}


type
  pbyte=^byte;
  {Definition of callback functions called by the DLL}
  {Ask to swap disk for multi-volume archive}
  PChangeVolProc=^TChangeVolProc;
  TChangeVolProc=function(ArcName:pchar;Mode:longint):longint; stdcall;
  {Notify that data is processed - used for progress dialog}
  PProcessDataProc=^TProcessDataProc;
  TProcessDataProc=function(Addr:pbyte;Size:longint):longint; stdcall;

type
  THeaderData=packed record
    ArcName:array [0..259] of char;
    FileName:array [0..259] of char;
    Flags,
    PackSize,
    UnpSize,
    HostOS,
    FileCRC,
    FileTime,
    UnpVer,
    Method,
    FileAttr:longint;
    CmtBuf:pchar;
    CmtBufSize,
    CmtSize,
    CmtState:longint;
  end;

  TOpenArchiveData=packed record
    ArcName:pchar;
    OpenMode,
    OpenResult:longint;
    CmtBuf:pchar;
    CmtBufSize,
    CmtSize,
    CmtState:longint;
  end;


const
  maxarchives = 10;

type
  ArchiveRec=record
    WS: TWebShots; //pointer to a TWebShots object
    CurrentPicIndex: Integer;
    ChangeVolProc:TChangeVolProc;
    ProcessDataProc:TProcessDataProc;
  end;

{Keep a list of currently open archives (up to a maximum of maxarchives)}
var
  ArchiveList: array[1..maxarchives] of ArchiveRec;

function OpenArchive(var ArchiveData:tOpenArchiveData):thandle; stdcall;
var
  FoundSlot: Boolean;
  i: Integer;
  ArcIdx: Integer;
begin
  Result := 0;
  ArcIdx := 0;
  ArchiveData.OpenResult := E_UNKNOWN_FORMAT;

  FoundSlot := False;
  for i := 1 to maxarchives do begin
    if ArchiveList[i].WS = nil then begin
      FoundSlot := True;
      ArcIdx := i;
    end;
  end;

  if FoundSlot then begin
    //try to open the file as wbc, wbz or wb1,wbd,wbo,spv
    try
      ArchiveList[ArcIdx].WS := TWebShots.Create(ArchiveData.ArcName);

      if ArchiveList[ArcIdx].WS.FileType = wsftUnknown then
        FreeAndNil(ArchiveList[ArcIdx].WS)
      else begin
        ArchiveList[ArcIdx].CurrentPicIndex := 1;
        ArchiveData.OpenResult := 0;
        Result := ArcIdx;
      end
    except
      FreeAndNil(ArchiveList[ArcIdx].WS);
    end;
  end
  else begin
    ArchiveData.OpenResult := E_NO_MEMORY;
  end
end;

function CloseArchive(hArcData:thandle):longint; stdcall;
begin
  if (hArcData > 0) and (hArcData <= maxarchives) then
    FreeAndNil(ArchiveList[hArcData].WS);
  Result:=0;
end;

function ReadHeader(hArcData:thandle;var HeaderData:THeaderData):longint; stdcall;
var
  Entry: TWSEntry;
begin
  if (hArcData > 0) and (hArcData <= maxarchives) and (ArchiveList[hArcData].WS <> nil) then begin
    FillChar(HeaderData,SizeOf(HeaderData),#0);

    if ArchiveList[hArcData].WS.GetEntry(ArchiveList[hArcData].CurrentPicIndex,Entry) then begin
      Inc(ArchiveList[hArcData].CurrentPicIndex);

      Move(Entry.FileName,HeaderData.FileName,SizeOf(HeaderData.FileName));
      HeaderData.PackSize := Entry.DataSize;
      HeaderData.UnpSize  := Entry.DataSize;
      HeaderData.FileTime := ((YearOf(Entry.Date) - 1980) shl 25) or (MonthOf(Entry.Date) shl 21) or (DayOf(Entry.Date) shl 16) or (HourOf(Entry.Date) shl 11) or (MinuteOf(Entry.Date) shl 5) or (SecondOf(Entry.Date) shl 1);

      Result := 0;
    end
    else //no more files
      Result := E_END_ARCHIVE
  end
	else
    Result := E_EREAD;
end;

function ProcessFile(hArcData:thandle;Operation:longint;DestPath,DestName:pchar):longint; stdcall;
var
  OutName: String;
begin
  Result := 0;
  if Operation = PK_EXTRACT then begin
    if (hArcData > 0) and (hArcData <= maxarchives) and (ArchiveList[hArcData].WS <> nil) then begin
      if DestPath = nil then
        OutName := DestName
      else
        OutName := String(DestPath) + String(DestName);

      if ArchiveList[hArcData].WS.Extract(ArchiveList[hArcData].CurrentPicIndex-1,OutName) then
        Result := 0
      else //no more files
        Result := E_BAD_DATA
    end
    else
      Result := E_EREAD;
  end;
end;

procedure SetChangeVolProc(hArcData:THandle;ChangeVolProc:TChangeVolProc); stdcall;
begin
  ArchiveList[hArcData].ChangeVolProc := ChangeVolProc;
end;

procedure SetProcessDataProc(hArcData:THandle;ProcessDataProc:TProcessDataProc); stdcall;
begin
  ArchiveList[hArcData].ProcessDataProc := ProcessDataProc;
end;

function GetPackerCaps:integer; stdcall;
begin
  Result := PK_CAPS_NEW or PK_CAPS_MODIFY or PK_CAPS_MULTIPLE or PK_CAPS_BY_CONTENT or PK_CAPS_SEARCHTEXT;
end;

function PackFiles(PackedFile,SubPath,SrcPath,AddList:PChar;Flags:integer):integer; stdcall;
var
  WS: TWebShots;
  AllOK: Boolean;
  p: PChar;
  FileName, Ext: String;
  FileType: Cardinal;
begin
  Result := E_EABORTED;

(*  if FileExists(PackedFile) then begin
    if MessageBox(GetActiveWindow,'File exists, overwrite?',PackedFile,mb_yesnocancel or mb_iconquestion) <> idYes then begin
      Result:=E_EABORTED;
      Exit
    end;
  end;
*)
  try
    //set desired file type to use if file doesn't exist and needs to be created
    Ext := AnsiUpperCase(ExtractFileExt(PackedFile));
    if (Ext = '.WBC') or (Ext = '.WBP') then
      FileType := wsftWBC
    else if (Ext = '.WB0') or (Ext = '.WBD') or (Ext = '.WBO') then
      FileType := wsftWBD
    else if Ext = '.WB1' then
      FileType := wsftWB1
    else if Ext = '.SPV' then
      FileType := wsftSPV
    else if Ext = '.WBZ' then
      FileType := wsftWBZ
    else //unknown extension, assume WB1
      FileType := wsftWB1;

    WS := TWebShots.Create(PackedFile, FileType);

    if WS.FileType = wsftUnknown then
      FreeAndNil(WS)
    else begin
      AllOK := True;
      p := AddList;

      while (p^ <> #0) and AllOK do begin
        FileName := String(SrcPath) + String(p);
        AllOK := AllOK and WS.Add(FileName);
        p := StrEnd(p)+1;
      end;

      if AllOK then
        Result := 0
      else
        Result := E_TOO_MANY_FILES;
    end
  finally
    FreeAndNil(WS);
  end;

end;

function DeleteFiles(PackedFile,DeleteList:pchar):integer; stdcall;
begin
  result:=E_NOT_SUPPORTED;
end;

function CanYouHandleThisFile(FileName: PChar): BOOL; stdcall;
var WS: TWebShots;
begin
  try
    WS := TWebShots.Create(FileName);
    Result := WS.FileType <> wsftUnknown;
    WS.Free
  except
    Result := False;
  end;
end;

exports
  OpenArchive,
  CloseArchive,
  ReadHeader,
  ProcessFile,
  GetPackerCaps,
  PackFiles,
  DeleteFiles,
  SetChangeVolProc,
  SetProcessDataProc,
  CanYouHandleThisFile;

begin
  FillChar(ArchiveList,sizeof(ArchiveList),#0);
end.
