library Webshots;

uses
  DateUtils,
  SysUtils,
  Classes,
  Messages,
  Windows,
  wcxhead,
  uWebshots in 'uWebshots.pas';

{$E wcx}
{$IFDEF WIN64}
  {$E w64}
  {$IFNDEF VER230}
    {$E wcx64}
  {$ENDIF}
{$ENDIF}

const
  maxarchives = 10;

type
  ArchiveRec=record
    WS: TWebShots; //pointer to a TWebShots object
    CurrentPicIndex: Integer;
    ChangeVolProc:TChangeVolProcW;
    ProcessDataProc:TProcessDataProcW;
  end;

{Keep a list of currently open archives (up to a maximum of maxarchives)}
var
  ArchiveList: array[1..maxarchives] of ArchiveRec;

function OpenArchive(var ArchiveData:tOpenArchiveData):THandle; stdcall;
begin
  Result := 0;
end;

function OpenArchiveW(var ArchiveData:tOpenArchiveDataW):THandle; stdcall;
var
  FoundSlot: Boolean;
  i: Integer;
  ArcIdx: Integer;
begin
  Result := 0;
  ArcIdx := 0;
  ArchiveData.OpenResult := E_UNKNOWN_FORMAT;

  FoundSlot := False;
  for i := 1 to maxarchives do begin
    if ArchiveList[i].WS = nil then begin
      FoundSlot := True;
      ArcIdx := i;
    end;
  end;

  if FoundSlot then begin
    //try to open the file as wbc, wbz or wb1,wbd,wbo,spv
    try
      ArchiveList[ArcIdx].WS := TWebShots.Create(ArchiveData.ArcName);

      if ArchiveList[ArcIdx].WS.FileType = wsftUnknown then
        FreeAndNil(ArchiveList[ArcIdx].WS)
      else begin
        ArchiveList[ArcIdx].CurrentPicIndex := 1;
        ArchiveData.OpenResult := 0;
        Result := ArcIdx;
      end
    except
      FreeAndNil(ArchiveList[ArcIdx].WS);
    end;
  end
  else begin
    ArchiveData.OpenResult := E_NO_MEMORY;
  end
end;

function CloseArchive(hArcData:thandle):longint; stdcall;
begin
  if (hArcData > 0) and (hArcData <= maxarchives) then
    FreeAndNil(ArchiveList[hArcData].WS);
  Result:=0;
end;

function ReadHeader(hArcData:THandle;var HeaderData:THeaderData):longint; stdcall;
begin
  Result := E_NOT_SUPPORTED;
end;

function ReadHeaderExW(hArcData:THandle;var HeaderData:THeaderDataExW):longint; stdcall;
var
  Entry: TWSEntry;
begin
  if (hArcData > 0) and (hArcData <= maxarchives) and (ArchiveList[hArcData].WS <> nil) then begin
    FillChar(HeaderData,SizeOf(HeaderData),#0);

    if ArchiveList[hArcData].WS.GetEntry(ArchiveList[hArcData].CurrentPicIndex,Entry) then begin
      Inc(ArchiveList[hArcData].CurrentPicIndex);

      StrLCopy(HeaderData.FileNameW,
        PWideChar(WideString(Entry.FileName)), Length(HeaderData.FileNameW) - 1);
      HeaderData.PackSize := Entry.DataSize;
      HeaderData.UnpSize  := Entry.DataSize;
      HeaderData.FileTime := ((YearOf(Entry.Date) - 1980) shl 25) or (MonthOf(Entry.Date) shl 21) or (DayOf(Entry.Date) shl 16) or (HourOf(Entry.Date) shl 11) or (MinuteOf(Entry.Date) shl 5) or (SecondOf(Entry.Date) shl 1);

      Result := 0;
    end
    else //no more files
      Result := E_END_ARCHIVE
  end
	else
    Result := E_EREAD;
end;

function ProcessFile(hArcData:THandle;Operation:longint;DestPath,DestName:pansichar):longint; stdcall;
begin
  Result := 0;
end;

function ProcessFileW(hArcData:THandle;Operation:longint;DestPath,DestName:PWideChar):longint; stdcall;
var
  OutName: String;
begin
  Result := 0;
  if Operation = PK_EXTRACT then begin
    if (hArcData > 0) and (hArcData <= maxarchives) and (ArchiveList[hArcData].WS <> nil) then begin
      if DestPath = nil then
        OutName := DestName
      else
        OutName := String(DestPath) + String(DestName);

      if ArchiveList[hArcData].WS.Extract(ArchiveList[hArcData].CurrentPicIndex-1,OutName) then
        Result := 0
      else //no more files
        Result := E_BAD_DATA
    end
    else
      Result := E_EREAD;
  end;
end;

procedure SetChangeVolProc(hArcData:THandle;ChangeVolProc:TChangeVolProc); stdcall;
begin

end;

procedure SetProcessDataProc(hArcData:THandle;ProcessDataProc:TProcessDataProc); stdcall;
begin

end;

procedure SetChangeVolProcW(hArcData:THandle;ChangeVolProc:TChangeVolProcW); stdcall;
begin
  ArchiveList[hArcData].ChangeVolProc := ChangeVolProc;
end;

procedure SetProcessDataProcW(hArcData:THandle;ProcessDataProc:TProcessDataProcW); stdcall;
begin
  ArchiveList[hArcData].ProcessDataProc := ProcessDataProc;
end;

function GetPackerCaps:integer; stdcall;
begin
  Result := PK_CAPS_NEW or PK_CAPS_MODIFY or PK_CAPS_MULTIPLE or PK_CAPS_BY_CONTENT or PK_CAPS_SEARCHTEXT;
end;

function PackFiles(PackedFile,SubPath,SrcPath,AddList:PAnsiChar;Flags:integer):integer; stdcall;
begin
  Result := E_NOT_SUPPORTED;
end;

function PackFilesW(PackedFile,SubPath,SrcPath,AddList:PChar;Flags:integer):integer; stdcall;
var
  WS: TWebShots;
  AllOK: Boolean;
  p: PChar;
  FileName, Ext: String;
  FileType: Cardinal;
begin
  Result := E_EABORTED;

(*  if FileExists(PackedFile) then begin
    if MessageBox(GetActiveWindow,'File exists, overwrite?',PackedFile,mb_yesnocancel or mb_iconquestion) <> idYes then begin
      Result:=E_EABORTED;
      Exit
    end;
  end;
*)
  try
    //set desired file type to use if file doesn't exist and needs to be created
    Ext := AnsiUpperCase(ExtractFileExt(PackedFile));
    if (Ext = '.WBC') or (Ext = '.WBP') then
      FileType := wsftWBC
    else if (Ext = '.WB0') or (Ext = '.WBD') or (Ext = '.WBO') then
      FileType := wsftWBD
    else if Ext = '.WB1' then
      FileType := wsftWB1
    else if Ext = '.SPV' then
      FileType := wsftSPV
    else if Ext = '.WBZ' then
      FileType := wsftWBZ
    else //unknown extension, assume WB1
      FileType := wsftWB1;

    WS := TWebShots.Create(PackedFile, FileType);
    try
      if WS.FileType = wsftUnknown then
        FreeAndNil(WS)
      else begin
        AllOK := True;
        p := AddList;

         while (p^ <> #0) and AllOK do
         begin
           FileName := String(SrcPath) + String(p);
           AllOK := AllOK and WS.Add(FileName);
           p := StrEnd(p)+1;
         end;

        if AllOK then
          Result := 0
        else
          Result := E_TOO_MANY_FILES;
      end
    finally
      FreeAndNil(WS);
    end;
  except
    Result := E_EWRITE;
  end;
end;

function DeleteFilesW(PackedFile,DeleteList:pchar):integer; stdcall;
begin
  result:=E_NOT_SUPPORTED;
end;

function CanYouHandleThisFileW(FileName: PChar): BOOL; stdcall;
var
  WS: TWebShots;
begin
  try
    WS := TWebShots.Create(FileName);
    try
      Result := WS.FileType <> wsftUnknown;
    finally
      WS.Free;
    end;
  except
    Result := False;
  end;
end;

exports
  OpenArchive,
  OpenArchiveW,
  CloseArchive,
  ReadHeader,
  ReadHeaderExW,
  ProcessFile,
  ProcessFileW,
  GetPackerCaps,
  PackFiles,
  PackFilesW,
  DeleteFilesW,
  SetChangeVolProc,
  SetProcessDataProc,
  SetChangeVolProcW,
  SetProcessDataProcW,
  CanYouHandleThisFileW;

begin
  FillChar(ArchiveList,sizeof(ArchiveList),#0);
end.
